@@ -65,6 +65,28 @@ enum class ModelQuantity {
     qBdot_ss,
     xBdot_ss,
     JSparseB_ss,
+    deltax,
+    deltasx,
+    deltaxB,
+    k,
+    p,
+    ts,
+    dJydy,
+    dJydy_matlab,
+    deltaqB,
+    dsigmaydp,
+    dsigmaydy,
+    dJydsigma,
+    dJydx,
+    dzdx,
+    dzdp,
+    dJrzdsigma,
+    dJrzdz,
+    dJzdsigma,
+    dJzdz,
+    drzdp,
+    drzdx,
+    dsigmazdp,
 };
 
 extern const std::map<ModelQuantity, std::string> model_quantity_to_str;
@@ -1869,7 +1891,7 @@ class Model : public AbstractModel, public ModelDimensions {
     /** vector of bools indicating whether state variables are to be assumed to
      * be positive */
     std::vector<bool> state_is_non_negative_;
-    
+
     /** Vector of booleans indicating the initial boolean value for every event trigger function. Events at t0
      * can only trigger if the initial value is set to `false`. Must be specified during model compilation by
      * setting the `initialValue` attribute of an event trigger. */
